\name{exception}
\alias{exception}
\alias{simple.exception}
\alias{raise.exception}
\alias{try.catch}
\alias{default.exception.handler}

\title{Simple Exception Handling Mechanism}
\description{
  A simple exception handling mechanism for R.  This is in no sense
  meant as a final implementation but rather as a test bed for trying out
  ideas for how this might work.
}

\usage{
    simple.exception(string)
    raise.exception(except)
    try.catch(expr, ..., finally = NULL)
    default.exception.handler(except)}

\arguments{
  \item{string}{ string describing the exception }
  \item{except}{exception object}
  \item{expr}{expression to execute for value}
  \item{finally}{expression to execute before returning or continuing a throw}
  \item{...}{named arguments with names representing exception classes and
             values that are functions of one argument} 
}

\details{
   Exceptions are objects inheriting from the abstract class \code{exception}.
   The class \code{simple.exception} is the class currently used by
   \code{stop} and all internal error signals.  The constructor by the
   same name takes a string describing the exception as argument and
   returns a \code{simple.exception} object.

   Uncaught exceptions will be processed by the generic function 
   \code{default.exception.handler}.  The method for class \code{exception}
   calls \code{stop} with the result of applying \code{as.character} to the
   exception argument.

   The function \code{try.catch} evaluates its expression argument
   in a context where the handlers provides in the \code{...} argument
   are available.  The \code{finally} expression is then evaluated in
   the context in which \code{try.catch} was called; that is, the
   handlers supplied to the current \code{try.catch} call are not
   active when the \code{finally} expression is evaluated.

   Handlers provided in the \code{...} argument to \code{try.catch}
   are established for the duration of the evaluation of \code{expr}.
   If no exception is raised when evaluating \code{expr} then
   \code{try.catch} returns the value of the expression.

   If an exception is raised while evaluating \code{expr} then
   established handlers are checked, starting with the most recently
   established ones, for one matching the class of the exception.  If
   a handler is found then control is transferred to the
   \code{try.catch} call that established the handler, the handlers in
   that call are disestablished, the handler is called with the
   exception as its argument, and the result returned by the handler is
   returned as the value of the \code{try.catch} call.

   If an exception is raised and no handler is found, then the default
   handler is called.  The context of the call is either the outer-most
   \code{try.catch} call or the \code{raise.exception} call if there is
   no surrounding \code{try.catch} call.

   When a \code{try.catch} call is on the stack calls to \code{stop} and
   errors signaled internally are converted into exceptions of type
   \code{simple.exception} and raised by \code{raise.exception}. (See the
   WARNING below on how this is done and its implications.)

   The only form of non-local transfer of control that \code{try.catch} 
   can catch is raising of exceptions.  It cannot capture the other
   three possibilities: calls to \code{return}, \code{break}, and
   \code{next}.  Whether these should be made, conceptually at least,
   into a kind of exception, or perhaps into something inheriting off a
   superclass of exception (in the spirit of Java's Throwable class) is
   not clear.  \code{try} and \code{restart} currently also do not catch
   these three, but it probably should.

   The printed representation of converted exceptions produced for
   \code{stop} calls and internal errors is less that ideal (as is the
   method of trapping and converting them) but this should do for getting
   a feel for how this might work.
}

\references{Similar to Java exceptions and exiting condition handlers
            in Dylan and Common Lisp}

\author{Luke Tierney}

\section{WARNING}{ At the moment the conversion of internal errors and
   errors signaled with \code{stop} works by hijacking the
   \code{error} option.  Setting this option in code inside a
   \code{try.catch} will have unpredictable consequences.
}

\examples{
try.catch(1, finally=print("Hello"))
e<-simple.exception("test exception")
\dontrun{raise.exception(e)}
\dontrun{try.catch(raise.exception(e), finally=print("Hello"))}
\dontrun{try.catch(stop("fred"), finally=print("Hello"))}
try.catch(raise.exception(e), exception = function(e) e, finally=print("Hello"))
try.catch(stop("fred"),  exception = function(e) e, finally=print("Hello"))
}
